"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
from test.unit.rules import BaseRuleTestCase

from cfnlint.rules.resources.lmbd.SnapStartEnabled import SnapStartEnabled
from cfnlint.rules.resources.lmbd.SnapStartSupported import SnapStartSupported


class TestSnapStartEnabled(BaseRuleTestCase):
    """Test Lambda SnapStart enabled"""

    def setUp(self):
        """Setup"""
        super(TestSnapStartEnabled, self).setUp()
        self.collection.register(SnapStartEnabled())
        self.collection.register(SnapStartSupported())
        self.success_templates = [
            "test/fixtures/templates/good/resources/lambda/snapstart-enabled.yaml"
        ]

    def test_file_positive(self):
        """Test Positive"""
        self.helper_file_positive()

    def test_file_negative(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/lambda/snapstart-enabled.yaml", 1
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from cfnlint.rules import CloudFormationLintRule, RuleMatch


class SnapStartSupported(CloudFormationLintRule):
    """Check if Lambda function using SnapStart has the correct runtimes"""

    id = "E2530"
    shortdesc = "SnapStart supports the configured runtime"
    description = (
        "To properly leverage SnapStart, you must have a runtime of Java11 or greater"
    )
    source_url = "https://docs.aws.amazon.com/lambda/latest/dg/snapstart.html"
    tags = ["resources", "lambda"]

    def __init__(self):
        super().__init__()
        self.resource_property_types.append("AWS::Lambda::Function")
        self.child_rules = {"I2530": None}

    def match_resource_properties(self, properties, _, path, cfn):
        """Check CloudFormation Properties"""
        matches = []

        for scenario in cfn.get_object_without_nested_conditions(properties, path):
            props = scenario.get("Object")

            runtime = props.get("Runtime")
            snap_start = props.get("SnapStart")
            if not snap_start:
                if self.child_rules["I2530"]:
                    matches.extend(self.child_rules["I2530"].validate(runtime, path))
                continue

            if snap_start.get("ApplyOn") != "PublishedVersions":
                continue

            # Validate runtime is a string before using startswith
            if not isinstance(runtime, str):
                continue

            if (
                runtime
                and (not runtime.startswith("java"))
                and runtime not in ["java8.al2", "java8"]
            ):
                matches.append(
                    RuleMatch(
                        path + ["SnapStart", "ApplyOn"],
                        f"{runtime} is not supported for SnapStart enabled functions",
                    )
                )

        return matches



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
from test.testlib.testcase import BaseTestCase

import cfnlint.core
from cfnlint.rules import CloudFormationLintRule, Match


class ErrorRule(CloudFormationLintRule):
    id = "E0000"
    description = "Error"


class WarningRule(CloudFormationLintRule):
    id = "W0000"
    description = "Warning"


class InformationRule(CloudFormationLintRule):
    id = "I0000"
    description = "Informational"


error_match = Match(0, 0, 0, 0, "", ErrorRule(), message="")
warning_match = Match(0, 0, 0, 0, "", WarningRule(), message="")
informational_match = Match(0, 0, 0, 0, "", InformationRule(), message="")


class TestExitCodes(BaseTestCase):
    """Test Exit Codes"""

    def test_exit_code_zero(self):
        params = [
            [[], "informational", 0],
            [[error_match], "informational", 2],
            [[error_match], "warning", 2],
            [[error_match], "error", 2],
            [[error_match], "none", 0],
            [[error_match, warning_match], "informational", 6],
            [[error_match, warning_match], "warning", 6],
            [[error_match, warning_match], "error", 2],
            [[error_match, warning_match], "none", 0],
            [[warning_match, informational_match], "informational", 12],
            [[warning_match, informational_match], "warning", 4],
            [[warning_match, informational_match], "error", 0],
            [[warning_match, informational_match], "none", 0],
            [[error_match, informational_match], "informational", 10],
            [[error_match, informational_match], "warning", 2],
            [[error_match, informational_match], "error", 2],
            [[error_match, informational_match], "none", 0],
            [[error_match, warning_match, informational_match], "informational", 14],
            [[error_match, warning_match, informational_match], "warning", 6],
            [[error_match, warning_match, informational_match], "error", 2],
            [[error_match, warning_match, informational_match], "none", 0],
        ]

        for matches, level, exit_code in params:
            with self.subTest():
                match_names = [x.rule.description for x in matches]
                self.assertEqual(
                    cfnlint.core.get_exit_code(matches, level),
                    exit_code,
                    f"{match_names} matches with {level} level should yield {exit_code}",
                )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""
from test.unit.rules import BaseRuleTestCase

from cfnlint.rules.functions.Split import Split  # pylint: disable=E0401


class TestRulesSplit(BaseRuleTestCase):
    """Test Rules Get Att"""

    def setUp(self):
        """Setup"""
        super(TestRulesSplit, self).setUp()
        self.collection.register(Split())

    def test_file_positive(self):
        """Test Positive"""
        self.helper_file_positive()

    def test_file_negative(self):
        """Test failure"""
        self.helper_file_negative("test/fixtures/templates/bad/functions_split.yaml", 4)

    def test_split_parts(self):
        rule = Split()
        self.assertEqual(len(rule._test_delimiter({}, [])), 1)
        self.assertEqual(len(rule._test_string({}, [])), 1)

        # supported function
        self.assertEqual(len(rule._test_string({"Ref": "Test"}, [])), 0)
        # Unsupported function
        self.assertEqual(len(rule._test_string({"Foo": "Bar"}, [])), 1)
